==> webform_gigya-1.0.0.tar.gz: webform_gigya/src/Plugin/WebformHandler/LiteAccountRegistrationHandler.php <==
<?php

namespace Drupal\webform_gigya\Plugin\WebformHandler;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\gigya\CmsStarterKit\sdk\GSApiException;
use Drupal\gigya\CmsStarterKit\sdk\GSObject;
use Drupal\gigya\CmsStarterKit\sdk\GSResponse;
use Drupal\gigya\Helper\GigyaHelper;
use Drupal\webform\Plugin\WebformElementManagerInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformMessageManagerInterface;
use Drupal\webform\WebformSubmissionConditionsValidatorInterface;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform\WebformTokenManagerInterface;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform submission lite account registration handler.
 *
 * @WebformHandler(
 *   id = "lite_account_registration",
 *   label = @Translation("Gigya Lite Account Registration"),
 *   category = @Translation("External"),
 *   description = @Translation("Call to Gigya API to register lite account."),
 *   tokens = TRUE,
 * )
 */
class LiteAccountRegistrationHandler extends WebformHandlerBase {

  const RESPONSE_DATA_DEFAULT = [
    'callId' => '',
    'errorCode' => '',
    'apiVersion' => '',
    'statusCode' => '',
    'statusReason' => '',
    'time' => '',
    'UID' => '',
  ];

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The token manager.
   *
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected $tokenManager;

  /**
   * The webform message manager.
   *
   * @var \Drupal\webform\WebformMessageManagerInterface
   */
  protected $messageManager;

  /**
   * A webform element plugin manager.
   *
   * @var \Drupal\webform\Plugin\WebformElementManagerInterface
   */
  protected $elementManager;

  /**
   * List of unsupported webform submission properties.
   *
   * The below properties will not being included in a remote post.
   *
   * @var array
   */
  protected $unsupportedProperties = [
    'metatag',
  ];

  /**
   * Drupal\gigya\Helper\GigyaHelper.
   *
   * @var \Drupal\gigya\Helper\GigyaHelper
   */
  protected $gigyaHelper;

  /**
   * Drupal\Core\Logger\LoggerChannelInterface.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, WebformSubmissionConditionsValidatorInterface $conditions_validator, ModuleHandlerInterface $module_handler, ClientInterface $http_client, WebformTokenManagerInterface $token_manager, WebformMessageManagerInterface $message_manager, WebformElementManagerInterface $element_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger_factory, $config_factory, $entity_type_manager, $conditions_validator);
    $this->moduleHandler = $module_handler;
    $this->httpClient = $http_client;
    $this->tokenManager = $token_manager;
    $this->messageManager = $message_manager;
    $this->elementManager = $element_manager;
    if (class_exists('Drupal\gigya\Helper\GigyaHelper')) {
      $this->gigyaHelper = new GigyaHelper();
    }
    else {
      $this->messenger->addError('Gigya module is missing.');
    }
    $this->logger = $this->getLogger('webform_gigya');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('webform_submission.conditions_validator'),
      $container->get('module_handler'),
      $container->get('http_client'),
      $container->get('webform.token_manager'),
      $container->get('webform.message_manager'),
      $container->get('plugin.manager.webform.element')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $configuration = $this->getConfiguration();
    $settings = $configuration['settings'];
    return [
        '#settings' => $settings,
      ] + parent::getSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'custom_data' => '',
      'debug' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    // Additional.
    $form['additional'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Additional settings'),
    ];
    $form['additional']['custom_data'] = [
      '#type' => 'webform_codemirror',
      '#mode' => 'yaml',
      '#title' => $this->t('Custom parameters'),
      '#description' => $this->t('Enter custom parameters that will be included in all remote post requests.'),
      '#default_value' => $this->configuration['custom_data'],
    ];

    // Development.
    $form['development'] = [
      '#type' => 'details',
      '#title' => $this->t('Development settings'),
    ];
    $form['development']['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable debugging'),
      '#description' => $this->t('If checked, posted submissions will be displayed onscreen to all users.'),
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['debug'],
    ];

    $form['handler_token'] = [
      '#type' => 'details',
      '#title' => $this->t('Handler response token'),
    ];
    $token = [];
    foreach (self::RESPONSE_DATA_DEFAULT as $key => $item) {
      $token[] = '[webform:handler:' . $this->getHandlerId() . ':' . $key . ']';
    }
    $form['handler_token']['title'] = [
      '#markup' => '<h3>' . $this->t('Use below token if you want to store response data:') . '</h3>',
    ];
    $form['handler_token']['token'] = [
      '#title' => $this->t('Use below token if you want to store response data:'),
      '#markup' => implode('<br/>', $token),
    ];

    $this->elementTokenValidate($form);

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    $this->sendRequest($webform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function postDelete(WebformSubmissionInterface $webform_submission) {
  }

  /**
   * Send request to Gigya.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   Drupal\webform\WebformSubmissionInterface.
   *
   * @throws \Exception
   */
  protected function sendRequest(WebformSubmissionInterface $webform_submission) {
    $data = $webform_submission->getData();
    $params = new GSObject();
    $params->put('isLite', TRUE);
    $response = $this->gigyaHelper->sendApiCall('accounts.initRegistration', $params);
    $this->debug($response, 'CALL accounts.initRegistration');
    if ($response instanceof GSApiException) {
      // Clear token.
      $this->saveResultAfterRequest($webform_submission, []);
      return;
    }
    $regToken = $response->getString('regToken');
    if (empty($regToken)) {
      $this->logger->error('Gigya regToken is missing');
    }

    // Get request options with tokens replaced.
    $request_options = (!empty($this->configuration['custom_data'])) ? Yaml::decode($this->configuration['custom_data']) : [];
    $request_options = $this->replaceTokens($request_options, $webform_submission);
    if (empty($request_options['regToken'])) {
      $request_options['regToken'] = $regToken;
    }
    if (empty($request_options['profile'])) {
      $request_options['profile'] = $data;
    }
    $params = new GSObject(Json::encode($request_options));
    $response = $this->gigyaHelper->sendApiCall('accounts.setAccountInfo', $params);
    $this->debug($response, 'CALL accounts.setAccountInfo');
    if ($response instanceof GSApiException) {
      // Clear token.
      $this->saveResultAfterRequest($webform_submission, []);
      return;
    }
    $gsObject = $response->getData();

    if (!empty($gsObject)) {
      $gigya_data = $gsObject->serialize();
    }
    $this->saveResultAfterRequest($webform_submission, $gigya_data);
  }

  /**
   * Save result of Gigya response.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   Drupal\webform\WebformSubmissionInterface.
   * @param array $gigya_data
   *   Gigya response.
   */
  protected function saveResultAfterRequest(WebformSubmissionInterface $webform_submission, array $gigya_data = []) {
    if (empty($gigya_data)) {
      $gigya_data = self::RESPONSE_DATA_DEFAULT;
    }
    // Replace [webform:handler] tokens in submission data.
    // Data structured for [webform:handler:remote_post:completed:key] tokens.
    $submission_data = $webform_submission->getData();
    $submission_has_token = (strpos(print_r($submission_data, TRUE), '[webform:handler:' . $this->getHandlerId() . ':') !== FALSE) ? TRUE : FALSE;
    if ($submission_has_token) {
      $token_data = ['webform_handler' => [$this->getHandlerId() => $gigya_data]];
      $submission_data = $this->replaceTokens($submission_data, $webform_submission, $token_data, ['clear' => TRUE]);
      $webform_submission->setData($submission_data);
      if ($this->isResultsEnabled()) {
        $webform_submission->resave();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function buildTokenTreeElement(array $token_types = [
    'webform',
    'webform_submission',
  ], $description = NULL) {
    $description = $description ?: $this->t('Use [webform_submission:values:ELEMENT_KEY:raw] to get plain text values.');
    return parent::buildTokenTreeElement($token_types, $description);
  }

  /**
   * Determine if saving of results is enabled.
   *
   * @return bool
   *   TRUE if saving of results is enabled.
   */
  protected function isResultsEnabled() {
    return ($this->getWebform()->getSetting('results_disabled') === FALSE);
  }

  /**
   * Debug message.
   *
   * @param \Drupal\gigya\CmsStarterKit\sdk\GSResponse $response
   *   Drupal\gigya\CmsStarterKit\sdk\GSResponse.
   * @param string $title
   *   Title.
   */
  private function debug(GSResponse $response, $title = '') {
    if (empty($this->configuration['debug'])) {
      return;
    }

    $build['response_body'] = [
      '#type' => 'item',
      '#wrapper_attributes' => ['style' => 'margin: 0'],
      '#title' => $title,
      'data' => [
        '#markup' => var_export($response, TRUE),
        '#prefix' => '<pre>',
        '#suffix' => '</pre>',
      ],
    ];
    $this->messenger()->addMessage(\Drupal::service('renderer')
      ->renderPlain($build), 'warning');
  }

}

==> webform_gigya-1.0.2.tar.gz: webform_gigya/src/Plugin/WebformHandler/LiteAccountRegistrationHandler.php <==
<?php

namespace Drupal\webform_gigya\Plugin\WebformHandler;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Serialization\Yaml;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\gigya\CmsStarterKit\sdk\GSApiException;
use Drupal\gigya\CmsStarterKit\sdk\GSObject;
use Drupal\gigya\CmsStarterKit\sdk\GSResponse;
use Drupal\gigya\Helper\GigyaHelper;
use Drupal\webform\Plugin\WebformElementManagerInterface;
use Drupal\webform\Plugin\WebformHandlerBase;
use Drupal\webform\WebformMessageManagerInterface;
use Drupal\webform\WebformSubmissionConditionsValidatorInterface;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform\WebformTokenManagerInterface;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Webform submission lite account registration handler.
 *
 * @WebformHandler(
 *   id = "lite_account_registration",
 *   label = @Translation("Gigya Lite Account Registration"),
 *   category = @Translation("External"),
 *   description = @Translation("Call to Gigya API to register lite account."),
 *   tokens = TRUE,
 * )
 */
class LiteAccountRegistrationHandler extends WebformHandlerBase {

  const RESPONSE_DATA_DEFAULT = [
    'callId' => '',
    'errorCode' => '',
    'apiVersion' => '',
    'statusCode' => '',
    'statusReason' => '',
    'time' => '',
    'UID' => '',
  ];

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The token manager.
   *
   * @var \Drupal\webform\WebformTokenManagerInterface
   */
  protected $tokenManager;

  /**
   * The webform message manager.
   *
   * @var \Drupal\webform\WebformMessageManagerInterface
   */
  protected $messageManager;

  /**
   * A webform element plugin manager.
   *
   * @var \Drupal\webform\Plugin\WebformElementManagerInterface
   */
  protected $elementManager;

  /**
   * List of unsupported webform submission properties.
   *
   * The below properties will not being included in a remote post.
   *
   * @var array
   */
  protected $unsupportedProperties = [
    'metatag',
  ];

  /**
   * Drupal\gigya\Helper\GigyaHelper.
   *
   * @var \Drupal\gigya\Helper\GigyaHelper
   */
  protected $gigyaHelper;

  /**
   * Drupal\Core\Logger\LoggerChannelInterface.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, LoggerChannelFactoryInterface $logger_factory, ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entity_type_manager, WebformSubmissionConditionsValidatorInterface $conditions_validator, ModuleHandlerInterface $module_handler, ClientInterface $http_client, WebformTokenManagerInterface $token_manager, WebformMessageManagerInterface $message_manager, WebformElementManagerInterface $element_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $logger_factory, $config_factory, $entity_type_manager, $conditions_validator);
    $this->moduleHandler = $module_handler;
    $this->httpClient = $http_client;
    $this->tokenManager = $token_manager;
    $this->messageManager = $message_manager;
    $this->elementManager = $element_manager;
    if (class_exists('Drupal\gigya\Helper\GigyaHelper')) {
      $this->gigyaHelper = new GigyaHelper();
    }
    else {
      $this->messenger->addError('Gigya module is missing.');
    }
    $this->logger = $this->getLogger('webform_gigya');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('logger.factory'),
      $container->get('config.factory'),
      $container->get('entity_type.manager'),
      $container->get('webform_submission.conditions_validator'),
      $container->get('module_handler'),
      $container->get('http_client'),
      $container->get('webform.token_manager'),
      $container->get('webform.message_manager'),
      $container->get('plugin.manager.webform.element')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSummary() {
    $configuration = $this->getConfiguration();
    $settings = $configuration['settings'];
    return [
        '#settings' => $settings,
      ] + parent::getSummary();
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'custom_data' => '',
      'debug' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

    // Additional.
    $form['additional'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Additional settings'),
    ];
    $form['additional']['custom_data'] = [
      '#type' => 'webform_codemirror',
      '#mode' => 'yaml',
      '#title' => $this->t('Custom parameters'),
      '#description' => $this->t('Enter custom parameters that will be included in all remote post requests.'),
      '#default_value' => $this->configuration['custom_data'],
    ];

    // Development.
    $form['development'] = [
      '#type' => 'details',
      '#title' => $this->t('Development settings'),
    ];
    $form['development']['debug'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable debugging'),
      '#description' => $this->t('If checked, PARAMS and RESPONSE will be dump() to screen.'),
      '#return_value' => TRUE,
      '#default_value' => $this->configuration['debug'],
    ];

    if (!function_exists('dump')) {
      $form['development']['debug']['#disabled'] = TRUE;
      $form['development']['debug']['#suffix'] = '<p class="messages messages--error">' . $this->t('Function dump() does not exist.') . '</p>';
    }

    $form['handler_token'] = [
      '#type' => 'details',
      '#title' => $this->t('Handler response token'),
    ];
    $token = [];
    foreach (self::RESPONSE_DATA_DEFAULT as $key => $item) {
      $token[] = '[webform:handler:' . $this->getHandlerId() . ':' . $key . ']';
    }
    $form['handler_token']['title'] = [
      '#markup' => '<h3>' . $this->t('Use below token if you want to store response data:') . '</h3>',
    ];
    $form['handler_token']['token'] = [
      '#title' => $this->t('Use below token if you want to store response data:'),
      '#markup' => implode('<br/>', $token),
    ];

    $this->elementTokenValidate($form);

    return $this->setSettingsParents($form);
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    $this->applyFormStateToConfiguration($form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave(WebformSubmissionInterface $webform_submission, $update = TRUE) {
    $this->sendRequest($webform_submission);
  }

  /**
   * {@inheritdoc}
   */
  public function postDelete(WebformSubmissionInterface $webform_submission) {
  }

  /**
   * Send request to Gigya.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   Drupal\webform\WebformSubmissionInterface.
   *
   * @throws \Exception
   */
  protected function sendRequest(WebformSubmissionInterface $webform_submission) {
    $data = $webform_submission->getData();
    $params = new GSObject();
    $params->put('isLite', TRUE);

    $this->debug($params, 'PARAMS accounts.initRegistration');
    $response = $this->gigyaHelper->sendApiCall('accounts.initRegistration', $params);
    $this->debug($response, 'RESPONSE accounts.initRegistration');

    if ($response instanceof GSApiException) {
      // Clear token.
      $this->saveResultAfterRequest($webform_submission, []);
      return;
    }
    $regToken = $response->getString('regToken');
    if (empty($regToken)) {
      $this->logger->error('Gigya regToken is missing');
    }

    // Get request options with tokens replaced.
    $request_options = (!empty($this->configuration['custom_data'])) ? Yaml::decode($this->configuration['custom_data']) : [];
    $request_options = $this->replaceTokens($request_options, $webform_submission);
    if (empty($request_options['regToken'])) {
      $request_options['regToken'] = $regToken;
    }
    if (empty($request_options['profile'])) {
      $request_options['profile'] = $data;
    }
    $params = new GSObject(Json::encode($request_options));

    $this->debug($params, 'PARAMS accounts.setAccountInfo');
    $response = $this->gigyaHelper->sendApiCall('accounts.setAccountInfo', $params);
    $this->debug($response, 'RESPONSE accounts.setAccountInfo');

    if ($response instanceof GSApiException) {
      // Clear token.
      $this->saveResultAfterRequest($webform_submission, []);
      return;
    }
    $gsObject = $response->getData();

    if (!empty($gsObject)) {
      $gigya_data = $gsObject->serialize();
    }
    $this->saveResultAfterRequest($webform_submission, $gigya_data);
  }

  /**
   * Save result of Gigya response.
   *
   * @param \Drupal\webform\WebformSubmissionInterface $webform_submission
   *   Drupal\webform\WebformSubmissionInterface.
   * @param array $gigya_data
   *   Gigya response.
   */
  protected function saveResultAfterRequest(WebformSubmissionInterface $webform_submission, array $gigya_data = []) {
    if (empty($gigya_data)) {
      $gigya_data = self::RESPONSE_DATA_DEFAULT;
    }
    // Replace [webform:handler] tokens in submission data.
    // Data structured for [webform:handler:remote_post:completed:key] tokens.
    $submission_data = $webform_submission->getData();
    $submission_has_token = (strpos(print_r($submission_data, TRUE), '[webform:handler:' . $this->getHandlerId() . ':') !== FALSE) ? TRUE : FALSE;
    if ($submission_has_token) {
      $token_data = ['webform_handler' => [$this->getHandlerId() => $gigya_data]];
      $submission_data = $this->replaceTokens($submission_data, $webform_submission, $token_data, ['clear' => TRUE]);
      $webform_submission->setData($submission_data);
      if ($this->isResultsEnabled()) {
        $webform_submission->resave();
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function buildTokenTreeElement(array $token_types = [
    'webform',
    'webform_submission',
  ], $description = NULL) {
    $description = $description ?: $this->t('Use [webform_submission:values:ELEMENT_KEY:raw] to get plain text values.');
    return parent::buildTokenTreeElement($token_types, $description);
  }

  /**
   * Determine if saving of results is enabled.
   *
   * @return bool
   *   TRUE if saving of results is enabled.
   */
  protected function isResultsEnabled() {
    return ($this->getWebform()->getSetting('results_disabled') === FALSE);
  }

  /**
   * Debug message.
   *
   * @param \Drupal\gigya\CmsStarterKit\sdk\GSResponse $response
   *   Drupal\gigya\CmsStarterKit\sdk\GSResponse.
   * @param string $title
   *   Title.
   */
  private function debug($response, $title = '') {
    if (empty($this->configuration['debug'])) {
      return;
    }
    dump($response);
  }

}
